# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ..core.datetime_utils import serialize_datetime
from ..core.pydantic_utilities import deep_union_pydantic_dicts, pydantic_v1
from .prompt_version import PromptVersion
from .refined_prompt_response_refinement_status import RefinedPromptResponseRefinementStatus


class RefinedPromptResponse(pydantic_v1.BaseModel):
    title: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    Title of the refined prompt
    """

    reasoning: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    Reasoning behind the refinement
    """

    prompt: str = pydantic_v1.Field()
    """
    The refined prompt text
    """

    refinement_job_id: typing.Optional[str] = pydantic_v1.Field(default=None)
    """
    Unique identifier for the refinement job
    """

    refinement_status: typing.Optional[RefinedPromptResponseRefinementStatus] = pydantic_v1.Field(default=None)
    """
    Status of the refinement job
    """

    total_cost: typing.Optional[float] = pydantic_v1.Field(default=None)
    """
    Total cost of the refinement job (in USD)
    """

    previous_version: typing.Optional[PromptVersion] = pydantic_v1.Field(default=None)
    """
    Previous version of the prompt
    """

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults_exclude_unset: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        kwargs_with_defaults_exclude_none: typing.Any = {"by_alias": True, "exclude_none": True, **kwargs}

        return deep_union_pydantic_dicts(
            super().dict(**kwargs_with_defaults_exclude_unset), super().dict(**kwargs_with_defaults_exclude_none)
        )

    class Config:
        frozen = True
        smart_union = True
        extra = pydantic_v1.Extra.allow
        json_encoders = {dt.datetime: serialize_datetime}
